# This file was auto-generated by Fern from our API Definition.

import typing

CompanyStatus = typing.Union[
    typing.Literal[
        "seized",
        "in_receivership",
        "registration_revoked",
        "opening",
        "dissolved",
        "active",
        "inactive",
        "closed",
        "closing",
        "registered",
        "incorporated",
        "under_external_control",
        "expired",
        "expanded",
        "terminated",
    ],
    typing.Any,
]
