# This file was auto-generated by Fern from our API Definition.

import typing
from .identifier_type import IdentifierType
from .weak_identifier_type import WeakIdentifierType

BothIdentifierTypes = typing.Union[IdentifierType, WeakIdentifierType]
