# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .psa_entity import PsaEntity
from .possibly_same_as_match import PossiblySameAsMatch
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class PossiblySameAsData(UniversalBaseModel):
    editable: typing.Optional[bool] = None
    entity: PsaEntity
    matches: typing.Dict[str, typing.List[PossiblySameAsMatch]]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
