# This file was auto-generated by Fern from our API Definition.

import typing

FinanceType = typing.Union[
    typing.Literal[
        "share_percentage",
        "share_amount",
        "registered_capital",
        "paid_up_capital",
        "authorized_capital",
        "subscribed_capital",
    ],
    typing.Any,
]
