# This file was auto-generated by Fern from our API Definition.

import typing

TranslationContext = typing.Union[
    typing.Literal["sayari_machine_translation", "pinyin", "google_translate"], typing.Any
]
