# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...base_types.types.paginated_response import PaginatedResponse
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from ...core.pydantic_utilities import update_forward_refs


class EntityRelationships(PaginatedResponse):
    """
    All relationships the entity is a part of.
    """

    next: typing.Optional[typing.Optional[typing.Any]] = None
    data: typing.List["RelationshipData"]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .entity_details import EntityDetails  # noqa: E402
from .relationship_data import RelationshipData  # noqa: E402

update_forward_refs(EntityDetails, EntityRelationships=EntityRelationships)
update_forward_refs(RelationshipData, EntityRelationships=EntityRelationships)
update_forward_refs(EntityRelationships)
